/*
 * kintone-mfinvoice-invoice.js
 * Copyright (c) 2021 StrategIT
 *
 * Licensed under the MIT License
 */
(function() {
  'use strict';
  var before_record;

  var tableDisabled = function(event) {
    var index, row;
    for (index = 0; index < event.record.items_aa.value.length; index++) {
      row = event.record.items_aa.value[index].value;
      row.name_item_code_aa.disabled = true;
      row.items_code_aa.disabled = true;
      row.price_aa.disabled = true;
      row.items_id_aa.disabled = true;
      row.items_name_aa.disabled = true;
      row.unit_aa.disabled = true;
      row.excise_aa.disabled = true;
      row.unit_price_aa.disabled = true;
      row.quantity_aa.disabled = true;
      row.detail_aa.disabled = true;
    }
    for (index = 0; index < event.record.items_lu.value.length; index++) {
      row = event.record.items_lu.value[index].value;
      row.items_code_lu.disabled = true;
      row.price_lu.disabled = true;
      row.items_id_lu.disabled = true;
      row.items_name_lu.disabled = false;
      row.unit_lu.disabled = false;
      row.excise_lu.disabled = false;
      row.unit_price_lu.disabled = false;
      row.quantity_lu.disabled = false;
      row.detail_lu.disabled = false;
    }
    return event;
  };

  var recordDisabled = function(event) {
    var record = event.record;
    record.name_suffix.disabled = true;
    record.zip.disabled = true;
    record.prefecture.disabled = true;
    record.address1.disabled = true;
    record.address2.disabled = true;
    record.department_name.disabled = true;
    record.person_title.disabled = true;
    record.person_name.disabled = true;
    record.partner_id.disabled = true;
    record.department_id.disabled = true;
    record.name_partner_copy.disabled = true;
    record.quotation_link.disabled = true;
    record.mf_invoice_link.disabled = true;
    record.partner_link.disabled = true;
  };

  var recordFieldShown = function(event) {
    kintone.app.record.setFieldShown('partner_id', false);
    kintone.app.record.setFieldShown('department_id', false);
    kintone.app.record.setFieldShown('name_partner_copy', false);
    kintone.app.record.setFieldShown('items_id', false);
    kintone.app.record.setGroupFieldOpen('billings', true);
  };

  var validateErrorClear = function(event) {
    var record = event.record;
    var index, row;
    event.error = '';
    record.note.error = '';
    record.payment_condition.error = '';
    record.billing_date.error = '';
    record.due_date.error = '';
    record.sales_date.error = '';
    for (index = 0; index < record.items_aa.value.length; index++) {
      row = record.items_aa.value[index].value;
      row.detail_aa.error = '';
      row.quantity_aa.error = '';
      row.unit_price_aa.error = '';
    }
    for (index = 0; index < record.items_lu.value.length; index++) {
      row = record.items_lu.value[index].value;
      row.detail_lu.error = '';
      row.quantity_lu.error = '';
      row.unit_price_lu.error = '';
    }
  };

  var validateRecord = function(event) {
    var record = event.record;
    if (record.note.value && record.note.value.length > 500) {
      record.note.error = '501文字より短くなければなりません。';
    }
    if (record.payment_condition.value && record.payment_condition.value.length > 250) {
      record.payment_condition.error = '251文字より短くなければなりません。';
    }
    if (record.billing_date.value && record.due_date.value &&
        record.billing_date.value >= record.due_date.value) {
      record.billing_date.error = '支払い期限より前の日付をセットしてください。';
    }
    if (record.note.error || record.payment_condition.error || record.billing_date.error) {
      event.error = '入力内容にエラーがあるため保存できません。';
    }
  };

  var validateTable = function(event) {
    var index, row;
    var totalQuantityMessage = '品目ごとの金額が -10,000,000,000 以上 10,000,000,000 以下になるよう入力してください。';
    var requiredMessage = '必須入力です。';
    var totalQuantity = 0;
    for (index = 0; index < event.record.items_lu.value.length; index++) {
      row = event.record.items_lu.value[index].value;
      if (row.detail_lu.value && row.detail_lu.value.length > 300) {
        row.detail_lu.error = '301文字より短くなければなりません。';
      }
      // 新規・編集用明細設定時のみ必須
      if (event.record.table_select.value === '新規・編集用明細') {
        if (!row.name_item_code_lu.value) {
          row.name_item_code_lu.error = requiredMessage;
        }
        if (!row.excise_lu.value) {
          row.excise_lu.error = requiredMessage;
        }
      }
      if (row.quantity_lu.value && row.unit_price_lu.value) {
        totalQuantity = row.quantity_lu.value * row.unit_price_lu.value;
        if (row.unit_lu.value === '%') {
          totalQuantity /= 100;
        }
        if (totalQuantity < -10000000000 || totalQuantity > 10000000000) {
          row.quantity_lu.error = totalQuantityMessage;
          row.unit_price_lu.error = totalQuantityMessage;
        }
      }
      if (row.detail_lu.error || row.quantity_lu.error || row.unit_price_lu.error || row.name_item_code_lu.error || row.excise_lu.error) {
        event.error = '入力内容にエラーがあるため保存できません。';
      }
    }
    for (index = 0; index < event.record.items_aa.value.length; index++) {
      row = event.record.items_aa.value[index].value;
      if (row.detail_aa.value && row.detail_aa.value.length > 300) {
        row.detail_aa.error = '301文字より短くなければなりません。';
      }
      // 見積書明細選択時のみ必須
      if (event.record.table_select.value === '見積書明細') {
        if (!row.name_item_code_aa.value) {
          row.name_item_code_aa.error = requiredMessage;
        }
        if (!row.excise_aa.value) {
          row.excise_aa.error = requiredMessage;
        }
      }
      if (row.quantity_aa.value && row.unit_price_aa.value) {
        totalQuantity = row.quantity_aa.value * row.unit_price_aa.value;
        if (row.unit_aa.value === '%') {
          totalQuantity /= 100;
        }
        if (totalQuantity < -10000000000 || totalQuantity > 10000000000) {
          row.quantity_aa.error = totalQuantityMessage;
          row.unit_price_aa.error = totalQuantityMessage;
        }
      }
      if (row.detail_aa.error || row.quantity_aa.error || row.unit_price_aa.error || row.name_item_code_aa.error || row.excise_aa.error) {
        event.error = '入力内容にエラーがあるため保存できません。';
      }
    }
  };

  var validateRequired = function(event) {
    var record = event.record;
    var billingsErrorMessage = '請求書の場合は、必須です。';
    if (!record.billing_date.value) {
      record.billing_date.error = billingsErrorMessage;
    }
    if (!record.due_date.value) {
      record.due_date.error = billingsErrorMessage;
    }
    if (!record.sales_date.value) {
      record.sales_date.error = billingsErrorMessage;
    }
    if (record.billing_date.error || record.due_date.error || record.sales_date.error) {
      event.error = '入力内容にエラーがあるため保存できません。';
      kintone.app.record.setGroupFieldOpen('billings', true);
    }
  };

  var toDateString = function(referenceDate) {
    var year = referenceDate.getFullYear();
    var month = ('0' + (referenceDate.getMonth() + 1)).slice(-2);
    var date = ('0' + referenceDate.getDate()).slice(-2);
    return year + '-' + month + '-' + date;
  };
  var selectDefaultValue = function(event) {
    var today = new Date();
    var endOfLastMonth, endOfNextMonth;
    var record = event.record;
    var next_month = today.getMonth() + 2;
    var last_month = today.getMonth();
    kintone.app.record.setGroupFieldOpen('billings', true);
    if (!record.billing_date.value) {
      record.billing_date.value = toDateString(today);
    }
    if (!record.due_date.value) {
      endOfNextMonth = new Date(today.getFullYear(), next_month, 0);
      record.due_date.value = toDateString(endOfNextMonth);
    }
    if (!record.sales_date.value) {
      endOfLastMonth = new Date(today.getFullYear(), last_month, 0);
      record.sales_date.value = toDateString(endOfLastMonth);
    }
    return event;
  };

  var tableColor = function(event) {
    var record = event.record;
    var items_aa = kintone.app.record.getFieldElement('items_aa');
    var items_lu = kintone.app.record.getFieldElement('items_lu');

    if (record.table_select.value === '見積書明細') {
      items_aa.classList.add('kintone-x-mfinvoice-select-table');
    } else {
      items_aa.classList.remove('kintone-x-mfinvoice-select-table');
    }
    if (record.table_select.value === '新規・編集用明細') {
      items_lu.classList.add('kintone-x-mfinvoice-select-table');
    } else {
      items_lu.classList.remove('kintone-x-mfinvoice-select-table');
    }
    return event;
  };

  var deleteTable = function(event) {
    var record = event.record;
    var invoice_id = record.レコード番号.value;
    var project_id = window.kintoneMfinvoiceInvoice.PROJECT_APP_ID;
    var get_param = {
      'app': project_id,
      'query': 'project_number = "' + record.project_number.value + '"',
      'fields': ['レコード番号', 'invoice_list']
    };
    return kintone.api(kintone.api.url('/k/v1/records.json', true), 'GET', get_param).then(function(project_records) {
      var update_param = {
        'app': project_id,
        'id': project_records.records[0].レコード番号.value,
        'record': {
          'invoice_list': {'value': []}
        }
      };
      var table = project_records.records[0].invoice_list.value;
      var i;
      for (i = 0; table.length > i; i++) {
        if (invoice_id !== table[i].value.invoice_id.value) {
          update_param.record.invoice_list.value.push({
            id: table[i].id
          });
        }
      }
      return update_param;
    }).then(function(update_param) {
      return kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', update_param);
    }).then(function() {
      return event;
    });
  };

  var partnerLink = function(event) {
    var partner_link_label;
    var partner_disp_link_label;
    var partner_link_display;
    var partner_link;
    var partner_disp_URL;
    var partner_record_id;
    partner_link_label = document.createElement('div');
    partner_disp_link_label = kintone.app.record.getSpaceElement('partner_link_display').appendChild(partner_link_label);
    partner_disp_link_label.id = 'partner_link_label';
    partner_disp_link_label.innerText = '取引先リンク';
    partner_link_display = document.createElement('a');
    partner_link = event.record.partner_link.value ? event.record.partner_link.value : '';
    if (partner_link) {
      partner_disp_URL = kintone.app.record.getSpaceElement('partner_link_display').appendChild(partner_link_display);
      partner_record_id = event.record.partner_id.value ? event.record.partner_id.value : '';
      partner_disp_URL.href = partner_link;
      partner_disp_URL.target = '_blank';
      partner_disp_URL.rel = 'noopener noreferrer';
      partner_disp_URL.innerText = '取引先#[' + partner_record_id + ']';
    }
    return event;
  };

  var quotationLink = function(event) {
    var quotation_link_label;
    var quotation_disp_link_label;
    var quotation_link_display;
    var quotation_link;
    var quotation_index;
    var partner_slice_link;
    var partner_slice_string;
    var quotation_record_id;
    var quotation_disp_URL;
    quotation_link_label = document.createElement('div');
    quotation_disp_link_label = kintone.app.record.getSpaceElement('quotation_link_display').appendChild(quotation_link_label);
    quotation_disp_link_label.id = 'quotation_link_label';
    quotation_disp_link_label.innerText = '見積書リンク';
    quotation_link_display = document.createElement('a');
    quotation_link = event.record.quotation_link.value ? event.record.quotation_link.value : '';
    if (quotation_link) {
      quotation_index = quotation_link.indexOf('/show#record=');
      partner_slice_link = quotation_link.slice(0, quotation_index);
      partner_slice_string = partner_slice_link + '/show#record=';
      quotation_record_id = quotation_link.replace(partner_slice_string, '');
      quotation_disp_URL = kintone.app.record.getSpaceElement('quotation_link_display').appendChild(quotation_link_display);
      quotation_disp_URL.href = quotation_link;
      quotation_disp_URL.target = '_blank';
      quotation_disp_URL.rel = 'noopener noreferrer';
      quotation_disp_URL.innerText = '見積書#[' + quotation_record_id + ']';
    }
    return event;
  };

  var mfInvoiceLink = function(event) {
    var mf_invoice_link_label;
    var mf_disp_invoice_link_label;
    var mf_invoice_link_display;
    var mf_invoice_link;
    var mf_disp_invoice_URL;
    mf_invoice_link_label = document.createElement('div');
    mf_disp_invoice_link_label = kintone.app.record.getSpaceElement('mf_invoice_link_display').appendChild(mf_invoice_link_label);
    mf_disp_invoice_link_label.id = 'mf_invoice_link_label';
    mf_disp_invoice_link_label.innerText = 'MF請求書リンク';
    mf_invoice_link_display = document.createElement('a');
    mf_invoice_link = event.record.mf_invoice_link.value ? event.record.mf_invoice_link.value : '';
    if (mf_invoice_link) {
      mf_disp_invoice_URL = kintone.app.record.getSpaceElement('mf_invoice_link_display').appendChild(mf_invoice_link_display);
      mf_disp_invoice_URL.href = mf_invoice_link;
      mf_disp_invoice_URL.target = '_blank';
      mf_disp_invoice_URL.rel = 'noopener noreferrer';
      mf_disp_invoice_URL.innerText = 'MF請求書';
    }
    return event;
  };

  var linkDisplay = function(event) {
    kintone.app.record.setFieldShown('partner_link', false);
    kintone.app.record.setFieldShown('quotation_link', false);
    kintone.app.record.setFieldShown('mf_invoice_link', false);
    quotationLink(event);
    mfInvoiceLink(event);
    partnerLink(event);
    return event;
  };

  var tableDisplay = function(event) {
    var record = event.record;
    if (record.table_select.value === '見積書明細') {
      kintone.app.record.setFieldShown('items_aa', true);
      kintone.app.record.setFieldShown('items_lu', false);
    } else if (record.table_select.value === '新規・編集用明細') {
      kintone.app.record.setFieldShown('items_aa', true);
      kintone.app.record.setFieldShown('items_lu', true);
    }
    return event;
  };

  var projectDelete = function(event) {
    var record = event.record;
    var invoice_id = record.レコード番号.value;
    var project_id = window.kintoneMfinvoiceInvoice.PROJECT_APP_ID;
    var invoice_project_number = record.project_number.value ? record.project_number.value : '';
    var project_project_number = before_record.project_number.value ? before_record.project_number.value : '';
    var i;
    var update_param = [];
    var table;
    var param = [];
    if (invoice_project_number === project_project_number) {
      return event;
    }
    param = {
      'app': project_id,
      'query': 'project_number = "' + project_project_number + '"',
      'fields': ['レコード番号', 'invoice_list']
    };
    return kintone.api(kintone.api.url('/k/v1/records.json', true), 'GET', param).then(function(project_records) {
      update_param = {
        'app': project_id,
        'id': project_records.records[0].レコード番号.value,
        'record': {
          'invoice_list': {'value': []}
        }
      };
      table = project_records.records[0].invoice_list.value;
      for (i = 0; table.length > i; i++) {
        if (invoice_id !== table[i].value.invoice_id.value) {
          update_param.record.invoice_list.value.push({
            id: table[i].id
          });
        }
      }
      return kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', update_param);
    }).then(function() {
      return event;
    });
  };

  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.index.edit.show'
  ], function(event) {
    before_record = event.record;
    return event;
  });

  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.index.edit.show'
  ], function(event) {
    recordDisabled(event);
    recordFieldShown(event);
    tableDisabled(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.change.items_lu',
    'app.record.create.change.items_aa',
    'app.record.edit.change.items_lu',
    'app.record.edit.change.items_aa',
    'app.record.index.edit.change.items_lu',
    'app.record.index.edit.change.items_aa'
  ], function(event) {
    tableDisabled(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.show',
    'app.record.create.change.status',
    'app.record.edit.change.status',
    'app.record.index.edit.change.status'
  ], function(event) {
    selectDefaultValue(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.submit',
    'app.record.edit.submit',
    'app.record.index.edit.submit'
  ], function(event) {
    validateErrorClear(event);
    validateRecord(event);
    validateTable(event);
    validateRequired(event);
    return event;
  });

  // リンクをテキスト表示
  kintone.events.on([
    'app.record.detail.show',
  ], function(event) {
    linkDisplay(event);
    return event;
  });

  // 新規単独登録時は「新規・編集用明細」、アプリアクションでの作成時は「見積書明細」
  kintone.events.on([
    'app.record.create.show'
  ], function(event) {
    var record = event.record;
    if (record.quotation_link.value) {
      record.table_select.value = '見積書明細';
    }
    return event;
  });

  // テーブルの表示/非表示
  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.create.change.table_select',
    'app.record.edit.change.table_select'
  ], function(event) {
    tableDisplay(event);
    tableDisabled(event);
    return event;
  });

  // 自動取得
  kintone.events.on([
    'app.record.create.show'
  ], function(event) {
    var record = event.record;
    // 案件番号
    record.project_number.lookup = true;
    return event;
  });

  // リンクをテキスト表示
  kintone.events.on([
    'app.record.detail.show',
  ], function(event) {
    tableColor(event);
    return event;
  });

  // 案件番号変更時案件レコードから削除
  kintone.events.on([
    'app.record.edit.submit'
  ], function(event) {
    projectDelete(event);
    return event;
  });

  // 請求書レコードが削除されたら案件アプリの請求書一覧のテーブルレコードも削除する
  kintone.events.on([
    'app.record.detail.delete.submit',
    'app.record.index.delete.submit'
  ], function(event) {
    deleteTable(event);
    return event;
  });
})();