/*
 * kintone-mfinvoice-target.js
 * Copyright (c) 2021 StrategIT
 *
 * Licensed under the MIT License
 */
(function() {
  'use strict';
  var before_record;

  var recordDisabled = function(event) {
    var record = event.record;
    record.name_suffix.disabled = true;
    record.zip.disabled = true;
    record.prefecture.disabled = true;
    record.address1.disabled = true;
    record.address2.disabled = true;
    record.department_name.disabled = true;
    record.person_title.disabled = true;
    record.person_name.disabled = true;
    record.partner_id.disabled = true;
    record.department_id.disabled = true;
    record.name_partner_copy.disabled = true;
    record.partner_link.disabled = true;
  };

  var tableClear = function(event) {
    var quotation_list = event.record.quotation_list;
    var invoice_list = event.record.invoice_list;
    // 新規作成時
    if (quotation_list.value.length === 1) {
      if (quotation_list.value[0].id === null) {
        quotation_list.value = [];
      }
    }
    // 新規作成時
    if (invoice_list.value.length === 1) {
      if (invoice_list.value[0].id === null) {
        invoice_list.value = [];
      }
    }
    // 編集時テーブルレコード0の時
    if (quotation_list.value.length === 0) {
      quotation_list.value = [];
    }
    if (invoice_list.value.length === 0) {
      invoice_list.value = [];
    }
  };
  var tableHide = function(event) {
    var quotation_list = event.record.quotation_list;
    var invoice_list = event.record.invoice_list;
    // 新規作成時
    if (quotation_list.value.length === 1) {
      if (quotation_list.value[0].id === null) {
        kintone.app.record.setFieldShown('quotation_list', false);
      }
    }
    // 新規作成時
    if (invoice_list.value.length === 1) {
      if (invoice_list.value[0].id === null) {
        kintone.app.record.setFieldShown('invoice_list', false);
      }
    }
    // 編集時テーブルレコード0の時
    if (quotation_list.value.length === 0) {
      kintone.app.record.setFieldShown('quotation_list', false);
    }
    if (invoice_list.value.length === 0) {
      kintone.app.record.setFieldShown('invoice_list', false);
    }
  };
  var editTableDisabled = function(event) {
    var index, row;
    for (index = 0; index < event.record.quotation_list.value.length; index++) {
      row = event.record.quotation_list.value[index].value;
      row.quotation_id.disabled = true;
    }
    for (index = 0; index < event.record.invoice_list.value.length; index++) {
      row = event.record.invoice_list.value[index].value;
      row.invoice_id.disabled = true;
    }
    return event;
  };

  var validateErrorClear = function(event) {
    var record = event.record;
    var index, row;
    event.error = '';
    record.memo.error = '';
    record.project_name.error = '';
    record.project_start_date.error = '';
    record.project_end_date.error = '';
    for (index = 0; index < record.quotation_list.value.length; index++) {
      row = record.quotation_list.value[index].value;
      row.quotation_id.error = '';
      row.invoice_start_date.error = '';
      row.invoice_end_date.error = '';
      row.invoice_create_date.error = '';
    }
    for (index = 0; index < record.invoice_list.value.length; index++) {
      row = record.invoice_list.value[index].value;
      row.invoice_id.error = '';
    }
  };

  var validateRecord = function(event) {
    var record = event.record;
    if (record.memo.value && record.memo.value.length > 500) {
      record.memo.error = '501文字より短くなければなりません。';
    }
    if (record.project_name.value && record.project_name.value.length > 250) {
      record.project_name.error = '251文字より短くなければなりません。';
    }
    if (record.project_start_date.value && record.project_end_date.value &&
      record.project_start_date.value >= record.project_end_date.value) {
      record.project_start_date.error = '案件終了日より前の日付をセットしてください。';
      record.project_end_date.error = '案件開始日より後の日付をセットしてください。';
    }
    if (record.memo.error || record.project_name.error || record.project_start_date.error || record.project_end_date.error) {
      event.error = '入力内容にエラーがあるため保存できません。';
    }
  };

  var validateTable = function(event) {
    var record = event.record;
    var index, row;
    var invoice_message = '請求ステータスが定期請求の場合は必須です';
    for (index = 0; index < record.quotation_list.value.length; index++) {
      row = record.quotation_list.value[index].value;
      if (row.invoice_start_date.value && row.invoice_end_date.value &&
        row.invoice_start_date.value > row.invoice_end_date.value) {
        row.invoice_start_date.error = '請求終了日より前の日付をセットしてください。';
        row.invoice_end_date.error = '請求開始日より後の日付をセットしてください。';
      }
      if (row.invoice_status.value === '定期請求') {
        if (!row.invoice_start_date.value) {
          row.invoice_start_date.error = invoice_message;
        }
        if (!row.invoice_end_date.value) {
          row.invoice_end_date.error = invoice_message;
        }
        if (!row.invoice_create_date.value) {
          row.invoice_create_date.error = invoice_message;
        }
      }
      if (!row.quotation_id.value) {
        row.quotation_id.error = '行の追加はできません。';
      }
      if (row.quotation_id.error) {
        event.error = 'テーブルの追加はできません。';
      } else if (row.invoice_start_date.error || row.invoice_end_date.error || row.invoice_create_date.error) {
        event.error = '入力内容にエラーがあるため保存できません。';
      }
    }
    for (index = 0; index < record.invoice_list.value.length; index++) {
      row = record.invoice_list.value[index].value;
      if (!row.invoice_id.value) {
        row.invoice_id.error = '行の追加はできません。';
      }
      if (row.invoice_id.error) {
        event.error = 'テーブルの追加はできません。';
      }
    }
  };

  var hideButton = function(event) {
    var quotationDispButton, invoiceDispButton, quotation_list_hide, invoice_list_hide, update_param;
    var record = event.record;
    // 非表示チェックボックスを非表示
    kintone.app.record.setFieldShown('quotation_list_hide', false);
    kintone.app.record.setFieldShown('invoice_list_hide', false);
    // 任意のスペースフィールドにボタンを設置
    quotationDispButton = document.createElement('button');
    quotationDispButton.id = 'quotation_button';
    quotationDispButton.innerText = '見積書非表示切り替え';
    invoiceDispButton = document.createElement('button');
    invoiceDispButton.id = 'invoice_button';
    invoiceDispButton.innerText = '請求書非表示切り替え';
    quotation_list_hide = record.quotation_list_hide.value ? record.quotation_list_hide.value : '';
    invoice_list_hide = record.invoice_list_hide.value ? record.invoice_list_hide.value : '';
    update_param = [];
    quotationDispButton.onclick = function() {
      quotationDispButton.disabled = true;
      if (quotation_list_hide.includes('非表示')) {
        quotation_list_hide = [];
        kintone.app.record.setFieldShown('quotation_list', true);
      } else {
        quotation_list_hide = ['非表示'];
        kintone.app.record.setFieldShown('quotation_list', false);
      }
      update_param = {
        'app': kintone.app.getId(),
        'id': event.record.レコード番号.value,
        'record': {
          'quotation_list_hide': {
            'value': quotation_list_hide
          }
        }
      };
      kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', update_param).then(function() {
        location.reload();
        return event;
      });
    };
    invoiceDispButton.onclick = function() {
      invoiceDispButton.disabled = true;
      if (invoice_list_hide.includes('非表示')) {
        invoice_list_hide = [];
        kintone.app.record.setFieldShown('invoice_list', true);
      } else {
        invoice_list_hide = ['非表示'];
        kintone.app.record.setFieldShown('invoice_list', false);
      }
      update_param = {
        'app': kintone.app.getId(),
        'id': event.record.レコード番号.value,
        'record': {
          'invoice_list_hide': {
            'value': invoice_list_hide
          }
        }
      };
      kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', update_param).then(function() {
        location.reload();
        return event;
      });
    };
    kintone.app.record.getSpaceElement('quotation_hide_button').appendChild(quotationDispButton);
    kintone.app.record.getSpaceElement('invoice_hide_button').appendChild(invoiceDispButton);
    return event;
  };

  var invoiceUpdateParam = function(event) {
    var index1, index2, row1, row2, temp_param;
    var update_param = [];
    for (index1 = 0; index1 < before_record.invoice_list.value.length; index1++) {
      row1 = before_record.invoice_list.value[index1].value;
      for (index2 = 0; index2 < event.record.invoice_list.value.length; index2++) {
        row2 = event.record.invoice_list.value[index2].value;
        if (row1.invoice_id.value === row2.invoice_id.value) {
          temp_param = [];
          if (row1.billing_date.value !== row2.billing_date.value) {
            temp_param.billing_date = row2.billing_date.value;
          }
          if (row1.due_date.value !== row2.due_date.value) {
            temp_param.due_date = row2.due_date.value;
          }
          if (row1.sales_date.value !== row2.sales_date.value) {
            temp_param.sales_date = row2.sales_date.value;
          }
          if (row1.deposit_status.value !== row2.deposit_status.value) {
            temp_param.deposit_status = row2.deposit_status.value;
          }
          if (Object.keys(temp_param).length !== 0) {
            temp_param.invoice_id = row2.invoice_id.value;
            temp_param.invoice_link = row2.invoice_link.value;
            update_param.push(temp_param);
          }
          temp_param = [];
        }
      }
    }
    return update_param;
  };

  var invoiceUpdate = function(event) {
    var update_param = [];
    var i, url, index, app_url, k_length, app_index, invoice_app_id, invoice;
    // 更新対象か調べる
    update_param = invoiceUpdateParam(event);
    if (update_param.length === 0) {
      return event;
    }
    // 更新対象のみ更新
    for (i = 0; update_param.length > i; i++) {
      url = update_param[i].invoice_link;
      index = url.indexOf('/show#record=');
      app_url = url.slice(0, index);
      k_length = '/k/'.length;
      app_index = app_url.indexOf('/k/') + k_length;
      invoice_app_id = app_url.slice(app_index);
      invoice = {
        'app': invoice_app_id,
        'id': update_param[i].invoice_id,
        'record': {}
      };
      if (update_param[i].billing_date) {
        invoice.record.billing_date = {'value': update_param[i].billing_date};
      }
      if (update_param[i].due_date) {
        invoice.record.due_date = {'value': update_param[i].due_date};
      }
      if (update_param[i].sales_date) {
        invoice.record.sales_date = {'value': update_param[i].sales_date};
      }
      if (update_param[i].deposit_status) {
        invoice.record.deposit_status = {'value': update_param[i].deposit_status};
      }
      return kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', invoice).then(function() {
        tableClear(event);
        return event;
      });
    }
    return event;
  };

  var linkDisplay = function(event) {
    var partner_link_label, partner_disp_link_label, partner_link_display, partner_link, partner_disp_URL, partner_record_id;
    kintone.app.record.setFieldShown('partner_link', false);

    partner_link_label = document.createElement('div');
    partner_disp_link_label = kintone.app.record.getSpaceElement('partner_link_display').appendChild(partner_link_label);
    partner_disp_link_label.id = 'partner_link_label';
    partner_disp_link_label.innerText = '取引先リンク';
    partner_link_display = document.createElement('a');
    partner_link = event.record.partner_link.value ? event.record.partner_link.value : '';
    if (partner_link) {
      partner_disp_URL = kintone.app.record.getSpaceElement('partner_link_display').appendChild(partner_link_display);
      partner_record_id = event.record.partner_id.value ? event.record.partner_id.value : '';
      partner_disp_URL.href = partner_link;
      partner_disp_URL.target = '_blank';
      partner_disp_URL.rel = 'noopener noreferrer';
      partner_disp_URL.innerText = '取引先#[' + partner_record_id + ']';
    }
    return event;
  };

  var tableDisplay = function(event) {
    var record = event.record;
    if (record.quotation_list_hide.value.includes('非表示')) {
      kintone.app.record.setFieldShown('quotation_list', false);
    } else {
      kintone.app.record.setFieldShown('quotation_list', true);
    }
    if (record.invoice_list_hide.value.includes('非表示')) {
      kintone.app.record.setFieldShown('invoice_list', false);
    } else {
      kintone.app.record.setFieldShown('invoice_list', true);
    }
    return event;
  };

  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.index.edit.show'
  ], function(event) {
    before_record = event.record;
    return event;
  });

  kintone.events.on([
    'app.record.create.submit',
    'app.record.edit.submit',
    'app.record.index.edit.submit'
  ], function(event) {
    tableClear(event);
    validateErrorClear(event);
    validateRecord(event);
    validateTable(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.show',
    'app.record.create.change.quotation_list',
  ], function(event) {
    tableHide(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.index.edit.show'
  ], function(event) {
    recordDisabled(event);
    return event;
  });
  kintone.events.on([
    'app.record.edit.show',
    'app.record.index.edit.show',
    'app.record.edit.change.quotation_list',
    'app.record.index.edit.change.quotation_list',
    'app.record.edit.change.invoice_list',
    'app.record.index.edit.change.invoice_list'
  ], function(event) {
    editTableDisabled(event);
    tableHide(event);
    return event;
  });

  // 請求書を更新
  kintone.events.on([
    'app.record.edit.submit',
    'app.record.index.edit.submit'
  ], function(event) {
    invoiceUpdate(event);
    return event;
  });

  // リンクをテキスト表示
  kintone.events.on([
    'app.record.detail.show',
  ], function(event) {
    linkDisplay(event);
    return event;
  });

  // テーブルの表示/非表示
  kintone.events.on([
    'app.record.detail.show'
  ], function(event) {
    tableDisplay(event);
    return event;
  });

  kintone.events.on('app.record.detail.show', function(event) {
    hideButton(event);
    return event;
  });

})();
