/*
 * kintone-mfinvoice-quotation.js
 * Copyright (c) 2021 StrategIT
 *
 * Licensed under the MIT License
 */
(function() {
  'use strict';
  var before_record;

  var tableDisabled = function(event) {
    var index, row;
    for (index = 0; index < event.record.items.value.length; index++) {
      row = event.record.items.value[index].value;
      row.items_code.disabled = true;
      row.price.disabled = true;
      row.items_id.disabled = true;
      row.items_name.disabled = false;
      row.unit.disabled = false;
      row.excise.disabled = false;
      row.unit_price.disabled = false;
      row.quantity.disabled = false;
      row.detail.disabled = false;
    }
    return event;
  };

  var recordDisabled = function(event) {
    var record = event.record;
    record.name_suffix.disabled = true;
    record.zip.disabled = true;
    record.prefecture.disabled = true;
    record.address1.disabled = true;
    record.address2.disabled = true;
    record.department_name.disabled = true;
    record.person_title.disabled = true;
    record.person_name.disabled = true;
    record.partner_id.disabled = true;
    record.department_id.disabled = true;
    record.name_partner_copy.disabled = true;
    record.mf_quotation_link.disabled = true;
    record.partner_link.disabled = true;
  };

  var recordFieldShown = function(event) {
    kintone.app.record.setFieldShown('partner_id', false);
    kintone.app.record.setFieldShown('department_id', false);
    kintone.app.record.setFieldShown('name_partner_copy', false);
    kintone.app.record.setFieldShown('items_id', false);
  };

  var validateErrorClear = function(event) {
    var record = event.record;
    var index, row;
    event.error = '';
    record.note.error = '';
    record.quote_date.error = '';
    for (index = 0; index < record.items.value.length; index++) {
      row = record.items.value[index].value;
      row.detail.error = '';
      row.quantity.error = '';
      row.unit_price.error = '';
    }
  };

  var validateRecord = function(event) {
    var record = event.record;
    if (record.note.value && record.note.value.length > 500) {
      record.note.error = '501文字より短くなければなりません。';
    }
    if (record.note.error) {
      event.error = '入力内容にエラーがあるため保存できません。';
    }
  };

  var validateTable = function(event) {
    var record = event.record;
    var index, row;
    var totalQuantityMessage = '品目ごとの金額が -10,000,000,000 以上 10,000,000,000 以下になるよう入力してください。';
    var totalQuantity = 0;
    for (index = 0; index < record.items.value.length; index++) {
      row = record.items.value[index].value;
      if (row.detail.value && row.detail.value.length > 300) {
        row.detail.error = '301文字より短くなければなりません。';
      }
      if (row.quantity.value && row.unit_price.value) {
        totalQuantity = row.quantity.value * row.unit_price.value;
        if (row.unit.value === '%') {
          totalQuantity /= 100;
        }
        if (totalQuantity < -10000000000 || totalQuantity > 10000000000) {
          row.quantity.error = totalQuantityMessage;
          row.unit_price.error = totalQuantityMessage;
        }
      }
      if (row.detail.error || row.quantity.error || row.unit_price.error) {
        event.error = '入力内容にエラーがあるため保存できません。';
      }
    }
  };

  var validateRequired = function(event) {
    var record = event.record;
    var quotesErrorMessage = '見積書の場合は、必須です。';
    if (!record.quote_date.value) {
      record.quote_date.error = quotesErrorMessage;
    }
    if (record.quote_date.error) {
      event.error = '入力内容にエラーがあるため保存できません。';
    }
  };

  var toDateString = function(referenceDate) {
    var year = referenceDate.getFullYear();
    var month = ('0' + (referenceDate.getMonth() + 1)).slice(-2);
    var date = ('0' + referenceDate.getDate()).slice(-2);
    return year + '-' + month + '-' + date;
  };
  var selectDefaultValue = function(event) {
    var today = new Date();
    var record = event.record;
    var endOfThisMonth;
    var oldQuoteDate = record.quote_date.value;
    if (!record.quote_date.value) {
      record.quote_date.value = toDateString(today);
    }
    // expired_dateだけ必須ではないため、oldQuoteDateも無ければ自動入力
    if (!record.expired_date.value && !oldQuoteDate) {
      endOfThisMonth = new Date(today.getFullYear(), today.getMonth() + 1, 0);
      record.expired_date.value = toDateString(endOfThisMonth);
    }
    return event;
  };

  var deleteTable = function(event) {
    var record = event.record;
    var quotation_id = record.レコード番号.value;
    var project_id = window.kintoneMfinvoiceInvoice.PROJECT_APP_ID;
    var get_param = {
      'app': project_id,
      'query': 'project_number = "' + record.project_number.value + '"',
      'fields': ['レコード番号', 'quotation_list']
    };
    var table, i;
    return kintone.api(kintone.api.url('/k/v1/records.json', true), 'GET', get_param).then(function(project_records) {
      var update_param = {
        'app': project_id,
        'id': project_records.records[0].レコード番号.value,
        'record': {
          'quotation_list': {'value': []}
        }
      };
      table = project_records.records[0].quotation_list.value;
      for (i = 0; table.length > i; i++) {
        if (quotation_id !== table[i].value.quotation_id.value) {
          update_param.record.quotation_list.value.push({
            id: table[i].id
          });
        }
      }
      return update_param;
    }).then(function(update_param) {
      return kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', update_param);
    }).then(function() {
      return event;
    });
  };

  var linkDisplay = function(event) {
    var mf_quotation_link_label, mf_disp_quotation_link_label, mf_quotation_link_display, mf_quotation_link;
    var mf_disp_quotation_URL, partner_link_label, partner_disp_link_label, partner_link_display, partner_link;
    var partner_disp_URL, partner_record_id;
    kintone.app.record.setFieldShown('partner_link', false);
    kintone.app.record.setFieldShown('mf_quotation_link', false);
    mf_quotation_link_label = document.createElement('div');
    mf_disp_quotation_link_label = kintone.app.record.getSpaceElement('mf_quotation_link_display').appendChild(mf_quotation_link_label);
    mf_disp_quotation_link_label.id = 'mf_quotation_link_label';
    mf_disp_quotation_link_label.innerText = 'MF請求書リンク';
    mf_quotation_link_display = document.createElement('a');
    mf_quotation_link = event.record.mf_quotation_link.value ? event.record.mf_quotation_link.value : '';
    if (mf_quotation_link) {
      mf_disp_quotation_URL = kintone.app.record.getSpaceElement('mf_quotation_link_display').appendChild(mf_quotation_link_display);
      mf_disp_quotation_URL.href = mf_quotation_link;
      mf_disp_quotation_URL.target = '_blank';
      mf_disp_quotation_URL.rel = 'noopener noreferrer';
      mf_disp_quotation_URL.innerText = 'MF見積書';
    }

    partner_link_label = document.createElement('div');
    partner_disp_link_label = kintone.app.record.getSpaceElement('partner_link_display').appendChild(partner_link_label);
    partner_disp_link_label.id = 'partner_link_label';
    partner_disp_link_label.innerText = '取引先リンク';
    partner_link_display = document.createElement('a');
    partner_link = event.record.partner_link.value ? event.record.partner_link.value : '';
    if (partner_link) {
      partner_disp_URL = kintone.app.record.getSpaceElement('partner_link_display').appendChild(partner_link_display);
      partner_record_id = event.record.partner_id.value ? event.record.partner_id.value : '';
      partner_disp_URL.href = partner_link;
      partner_disp_URL.target = '_blank';
      partner_disp_URL.rel = 'noopener noreferrer';
      partner_disp_URL.innerText = '取引先#[' + partner_record_id + ']';
    }
    return event;
  };

  var projectDelete = function(event) {
    var record = event.record;
    var quotation_id = record.レコード番号.value;
    var project_id = window.kintoneMfinvoiceInvoice.PROJECT_APP_ID;
    var quotation_project_number = record.project_number.value ? record.project_number.value : '';
    var project_project_number = before_record.project_number.value ? before_record.project_number.value : '';
    var old_project_number, new_project_number, old_param, update_param;
    var invoice_status, invoice_start_date, invoice_end_date, invoice_create_date, billing_date_setting;
    var due_date_setting, sales_date_setting, quotation_link, table, i, j, new_param;
    var projectUpdate = function() {
      return kintone.api(kintone.api.url('/k/v1/records.json', true), 'GET', new_param, function(new_project_records) {
        var new_update_param = {
          'app': project_id,
          'id': new_project_records.records[0].レコード番号.value,
          'record': {
            'quotation_list': {'value': []}
          }
        };
        var new_table = new_project_records.records[0].quotation_list.value;
        for (j = 0; new_table.length > j; j++) {
          if (quotation_id !== new_table[j].value.quotation_id.value) {
            new_update_param.record.quotation_list.value.push({
              id: new_table[j].id
            });
          }
        }
        // テーブルの値を追加
        new_update_param.record.quotation_list.value.push({
          value: {
            quotation_id: {value: quotation_id ? quotation_id : ''},
            invoice_status: {value: invoice_status ? invoice_status : '一回請求'},
            invoice_start_date: {value: invoice_start_date},
            invoice_end_date: {value: invoice_end_date},
            invoice_create_date: {value: invoice_create_date ? invoice_create_date : '月初'},
            billing_date_setting: {value: billing_date_setting ? billing_date_setting : '当日'},
            due_date_setting: {value: due_date_setting ? due_date_setting : '来月末'},
            sales_date_setting: {value: sales_date_setting ? sales_date_setting : '先月末'},
            quotation_link: {
              value: quotation_link ? quotation_link :
                window.location.origin + window.location.pathname + '#record=' + quotation_id
            }
          }
        });
        return kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', new_update_param);
      });
    };
    if (quotation_project_number === project_project_number) {
      return event;
    }
    old_project_number = before_record.project_number.value ? before_record.project_number.value : '';
    new_project_number = record.project_number.value ? record.project_number.value : '';
    // 案件番号が変わってなければ何もしない
    if (old_project_number === new_project_number) {
      return event;
    }
    old_param = {
      'app': project_id,
      'query': 'project_number = "' + old_project_number + '"',
      'fields': ['レコード番号', 'quotation_list']
    };
    update_param = {
      'app': project_id,
      'id': '',
      'record': {
        'quotation_list': {'value': []}
      }
    };
    new_param = {
      'app': project_id,
      'query': 'project_number = "' + new_project_number + '"',
      'fields': ['レコード番号', 'quotation_list']
    };
    // 元のレコードから削除
    if (old_project_number) {
      return kintone.api(kintone.api.url('/k/v1/records.json', true), 'GET', old_param).then(function(project_records) {
        update_param.id = project_records.records[0].レコード番号.value;
        table = project_records.records[0].quotation_list.value;
        for (i = 0; table.length > i; i++) {
          if (quotation_id !== table[i].value.quotation_id.value) {
            update_param.record.quotation_list.value.push({
              id: table[i].id
            });
          } else {
            invoice_status = table[i].value.invoice_status ? table[i].value.invoice_status.value : '';
            invoice_start_date = table[i].value.invoice_start_date ? table[i].value.invoice_start_date.value : '';
            invoice_end_date = table[i].value.invoice_end_date ? table[i].value.invoice_end_date.value : '';
            invoice_create_date = table[i].value.invoice_create_date ? table[i].value.invoice_create_date.value : '';
            billing_date_setting = table[i].value.billing_date_setting ? table[i].value.billing_date_setting.value : '';
            due_date_setting = table[i].value.due_date_setting ? table[i].value.due_date_setting.value : '';
            sales_date_setting = table[i].value.sales_date_setting ? table[i].value.sales_date_setting.value : '';
            quotation_link = table[i].value.quotation_link ? table[i].value.quotation_link.value : '';
          }
        }
        // 新しいレコードに追加
        if (new_project_number) {
          return projectUpdate();
        }
        return event;
      }).then(function() {
        return kintone.api(kintone.api.url('/k/v1/record.json', true), 'PUT', update_param);
      }).then(function() {
        return event;
      });
    } else if (new_project_number) {
      return projectUpdate();
    }
    return event;
  };

  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.index.edit.show'
  ], function(event) {
    before_record = event.record;
    return event;
  });


  kintone.events.on([
    'app.record.create.show',
    'app.record.edit.show',
    'app.record.index.edit.show'
  ], function(event) {
    recordDisabled(event);
    recordFieldShown(event);
    tableDisabled(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.change.items',
    'app.record.edit.change.items',
    'app.record.index.edit.change.items'
  ], function(event) {
    tableDisabled(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.show',
    'app.record.create.change.status',
    'app.record.edit.change.status',
    'app.record.index.edit.change.status'
  ], function(event) {
    selectDefaultValue(event);
    return event;
  });

  kintone.events.on([
    'app.record.create.submit',
    'app.record.edit.submit',
    'app.record.index.edit.submit'
  ], function(event) {
    validateErrorClear(event);
    validateRecord(event);
    validateTable(event);
    validateRequired(event);
    return event;
  });

  // 見積書レコードが削除されたら案件アプリの見積書一覧のテーブルレコードも削除する
  kintone.events.on([
    'app.record.detail.delete.submit',
    'app.record.index.delete.submit'
  ], function(event) {
    deleteTable(event);
  });

  // リンクをテキスト表示
  kintone.events.on([
    'app.record.detail.show',
  ], function(event) {
    linkDisplay(event);
    return event;
  });


  // 案件番号変更時案件レコードから削除
  kintone.events.on(['app.record.edit.submit'], function(event) {
    projectDelete(event);
    return event;
  });

  // 自動取得
  kintone.events.on([
    'app.record.create.show'
  ], function(event) {
    var record = event.record;
    // 案件番号
    record.project_number.lookup = true;
    return event;
  });

})();
